@@ -4,6 +4,7 @@ Types for high level PKCS#11 wrapper.
 This module provides stubs that are overrideen in pkcs11._pkcs11.
 """
 
+from threading import RLock
 from binascii import hexlify
 
 from .constants import *
@@ -177,6 +178,11 @@ class Session:
         """:class:`Token` this session is on."""
 
         self._handle = handle
+        # Big operation lock prevents other threads from entering/reentering
+        # operations. If the same thread enters the lock, they will get a
+        # Cryptoki warning
+        self._operation_lock = RLock()
+
         self.rw = rw
         """True if this is a read/write session."""
         self.user_type = user_type
@@ -186,6 +192,9 @@ class Session:
         return self.token == other.token and \
             self._handle == other._handle
 
+    def __hash__(self):
+        return hash(self._handle)
+
     def __enter__(self):
         return self
 
@@ -253,7 +262,7 @@ class Session:
             # collection, so that we release the operation lock.
             iterator._finalize()
 
-    def get_objects(self, attrs):
+    def get_objects(self, attrs=None):
         """
         Search for objects matching `attrs`. Returns a generator.
 
@@ -274,6 +283,32 @@ class Session:
         """
         raise NotImplementedError()
 
+    def create_object(self, attrs):
+        """
+        Create a new object on the :class:`Token`. This is a low-level
+        interface to create any type of object and can be used for importing
+        data onto the Token.
+
+        ::
+
+            key = session.create_object({
+                pkcs11.Attribute.CLASS: pkcs11.ObjectClass.SECRET_KEY,
+                pkcs11.Attribute.KEY_TYPE: pkcs11.KeyType.AES,
+                pkcs11.Attribute.VALUE: b'SUPER SECRET KEY',
+            })
+
+        For generating keys see :meth:`generate_key` or
+        :meth:`generate_keypair`.
+        For importing keys see :meth:`import_key` or :meth:`import_keypair`.
+
+        Requires a read/write session, unless the object is not to be
+        stored.
+
+        :param dict(Attribute,*) attrs: attributes of the object to create
+        :rtype: Object
+        """
+        raise NotImplementedError()
+
     def generate_key(self, key_type, key_length,
                      id=None, label=None,
                      store=True, capabilities=None,
@@ -343,11 +378,40 @@ class Object:
         return self.session == other.session and \
             self._handle == other._handle
 
+    def __hash__(self):
+        return hash((self.session, self._handle))
+
+    def copy(self, attrs):
+        """
+        Make a copy of the object with new attributes `attrs`.
+
+        Requires a read/write session, unless the object is not to be
+        stored.
+
+        ::
+
+            new = key.copy({
+                Attribute.LABEL: 'MY NEW KEY',
+            })
+
+        Certain objects may not be copied. Calling :meth:`copy` on such
+        objects will result in an exception.
+
+        :param dict(Attribute,*) attrs: attributes for the new :class:`Object`
+        :rtype: Object
+        """
+        raise NotImplementedError()
+
     def destroy(self):
         """
         Destroy the object.
 
-        Requires a R/W session.
+        Requires a read/write session, unless the object is not stored.
+
+        Certain objects may not be destroyed. Calling :meth:`destroy` on such
+        objects will result in an exception.
+
+        The :class:`Object` is no longer valid.
         """
         raise NotImplementedError()
 
@@ -401,7 +465,7 @@ class EncryptMixin(Object):
     def _encrypt(self, data, mechanism=None, mechanism_param=b''):
         raise NotImplementedError()
 
-    def encrypt(self, data, buffer_size=1024, **kwargs):
+    def encrypt(self, data, buffer_size=8192, **kwargs):
         """
         Encrypt some `data`.
 
@@ -425,6 +489,17 @@ class EncryptMixin(Object):
 
         Large single blocks of `data` will be chunked into the `buffer_size`.
 
+        The returned generator obtains a lock on the :class:`Session`
+        to prevent other threads from starting a simultaneous operation.
+        The lock is released when you consume/destroy the generator.
+        See :ref:`concurrency`.
+
+        .. warning::
+
+            It's not currently possible to cancel an encryption operation
+            by deleting the generator. You must consume the generator to
+            complete the operation.
+
         :param data: data to encrypt
         :type data: str, bytes or iter(bytes)
         :param Mechanism mechanism: optional encryption mechanism
@@ -447,7 +522,7 @@ class EncryptMixin(Object):
                                           buffer_size=buffer_size, **kwargs))
 
         else:
-            return self._encrypt(data, **kwargs)
+            return self._encrypt(data, buffer_size=buffer_size, **kwargs)
 
 
 class DecryptMixin(Object):
@@ -458,7 +533,7 @@ class DecryptMixin(Object):
     def _decrypt(self, data, mechanism=None, mechanism_param=b''):
         raise NotImplementedError()
 
-    def decrypt(self, data, buffer_size=1024, **kwargs):
+    def decrypt(self, data, buffer_size=8192, **kwargs):
         """
         Decrypt some `data`.
 
@@ -482,7 +557,7 @@ class DecryptMixin(Object):
                                           buffer_size=buffer_size, **kwargs))
 
         else:
-            return self._decrypt(data, **kwargs)
+            return self._decrypt(data, buffer_size=buffer_size, **kwargs)
 
 
 class SignMixin(Object):
